using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace WindowsGame1
{
    /// <summary>
    /// 基底 Game クラスから派生した、ゲームのメイン クラスです。
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {

        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        /// <summary>
        /// アニメーションテクスチャ
        /// </summary>
        AnimatedTexture2D starAnimationTexture1;

        /// <summary>
        /// アニメーションテクスチャ
        /// </summary>
        AnimatedTexture2D starAnimationTexture2;

        bool SpaceKeyIsDown = false;
        bool UpKeyIsDown = false;
        bool DownKeyIsDown = false;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// ゲームが実行を開始する前に必要な初期化を行います。
        /// ここで、必要なサービスを照会して、関連するグラフィック以外のコンテンツを
        /// 読み込むことができます。base.Initialize を呼び出すと、使用するすべての
        /// コンポーネントが列挙されるとともに、初期化されます。
        /// </summary>
        protected override void Initialize()
        {
            // TODO: ここに初期化ロジックを追加します。

            base.Initialize();
        }

        /// <summary>
        /// LoadContent はゲームごとに 1 回呼び出され、ここですべてのコンテンツを
        /// 読み込みます。
        /// </summary>
        protected override void LoadContent()
        {
            // 新規の SpriteBatch を作成します。これはテクスチャーの描画に使用できます。
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: this.Content クラスを使用して、ゲームのコンテンツを読み込みます。
            starAnimationTexture1 = new AnimatedTexture2D(Content.Load<Texture2D>("StarAnimation1"), 1, 9, 18);
            starAnimationTexture2 = new AnimatedTexture2D(Content.Load<Texture2D>("StarAnimation2"), 3, 3, 18);
        }

        /// <summary>
        /// UnloadContent はゲームごとに 1 回呼び出され、ここですべてのコンテンツを
        /// アンロードします。
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: ここで ContentManager 以外のすべてのコンテンツをアンロードします。
        }

        /// <summary>
        /// ワールドの更新、衝突判定、入力値の取得、オーディオの再生などの
        /// ゲーム ロジックを、実行します。
        /// </summary>
        /// <param name="gameTime">ゲームの瞬間的なタイミング情報</param>
        protected override void Update(GameTime gameTime)
        {
            // ゲームの終了条件をチェックします。
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: ここにゲームのアップデート ロジックを追加します。

            //スペースキーで一時停止と再開
            if (!SpaceKeyIsDown && Keyboard.GetState().IsKeyDown(Keys.Space))
            {
                if (!starAnimationTexture1.IsPaused)
                    starAnimationTexture1.Pause();
                else
                    starAnimationTexture1.Play();
            }

            //上下キーで速度調整
            if (!UpKeyIsDown && Keyboard.GetState().IsKeyDown(Keys.Up))
            {
                starAnimationTexture1.SetFramePerSecond(starAnimationTexture1.GetFramePerSecond() * 2F);
            }
            if (!DownKeyIsDown && Keyboard.GetState().IsKeyDown(Keys.Down))
            {
                starAnimationTexture1.SetFramePerSecond(starAnimationTexture1.GetFramePerSecond() * 0.5F);
            }

            //ウィンドウタイトルに現在の速度比を表示
            Window.Title = "StarAnimationFPS:"+starAnimationTexture1.GetFramePerSecond().ToString();

            //キー入力の更新
            UpKeyIsDown = Keyboard.GetState().IsKeyDown(Keys.Up);
            DownKeyIsDown = Keyboard.GetState().IsKeyDown(Keys.Down);
            SpaceKeyIsDown = Keyboard.GetState().IsKeyDown(Keys.Space);

            //アニメーションの更新
            starAnimationTexture1.Update((float)gameTime.ElapsedGameTime.TotalSeconds);
            starAnimationTexture2.Update((float)gameTime.ElapsedGameTime.TotalSeconds);

            base.Update(gameTime);
        }

        /// <summary>
        /// ゲームが自身を描画するためのメソッドです。
        /// </summary>
        /// <param name="gameTime">ゲームの瞬間的なタイミング情報</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.DarkBlue);

            // TODO: ここに描画コードを追加します。

            spriteBatch.Begin();

            //アニメーションの現在のフレームの描画
            starAnimationTexture1.DrawFrame(spriteBatch, new Vector2(100,100));
            starAnimationTexture2.DrawFrame(spriteBatch, new Vector2(300, 100));

            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
